;;; -*- auto-recompile: t -*-

;;; csound-key.el --- interfacing Csound-X with KeyKit

;; Keywords: csound

;; This file is not part of GNU Emacs.
;; 
;; csound-key.el is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; csound-key.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Author: Stphane Rollandin <hepta@zogotounga.net>
;;---------

;;; Commentary:
;;-------------

;;    full documentation here:
;;    http://www.zogotounga.net/comp/csoundx.html

;;
;;; Installation:
;;                  
;; ==========================================================
;; this file should be installed through the csound-x package
;; ==========================================================
;;

;; last modified April 14, 2005

;;; Code:

(require 'keykit-interaction nil t)


;;;;;;;;;  gnraliser: pas seulement Keykit, Squeak
;;;; et peut-tre d'autres formats (abc ?)
;;;; en bref: rationaliser, modulariser


(defcustom cscsd-midi-options "-W -TF "
  "Options for processing a midi file generated by a <KeyPhrase> area"
  :type 'string
  :group 'csound-csd)

(defcustom cscsd-key-decay 3
  "Decay time in seconds to be added to the duration of a KeyPhrase-generated MIDI file"
  :type 'integer
  :group 'csound-csd)

(defun cscsd-insert-KeyPhrase (&optional name)
  "Insert a <KeyPhrase> area at the beginning of the buffer, associated to keyword NAME"
  (interactive "sKeyPhrase name: ")
  (cscsd-go-before-cssynth)
  (save-excursion
    (unless (search-forward "MidiPhraseDuration" nil t)
      (goto-char
       (cscsd-define-macro "MidiPhraseDuration" 'sco ""))
      (insert ";f0 $MidiPhraseDuration.\n")
      (forward-line -2)
      (insert ";")))
  (insert "<KeyPhrase label=\"" name "\">\n"
	  "KeyPhrase=")
  (save-excursion 
    (insert  "''" "\n" "</KeyPhrase>\n\n"))
  (when (and (boundp mmm-mode) mmm-mode)
      (cscsd-turn-mmm-on)))

(defun cscsd-process-KeyPhrase (keyphrase &optional audioname)
  "Evaluate the KeyPhrase area, write its corresponding midifile
and process the CSD on it"
  (let* ((rootname (make-temp-file "cscsd-lowkey"))
	 (midifile (concat rootname ".mid"))
	 (durfile (concat rootname ".txt"))
	 (csdfile (concat rootname ".csd"))
	 audiofile ;; we need this since cscsd-latest-audio-file is buffer-local
	 (this-buffer (current-buffer))
	 (csdname (file-name-sans-extension 
		   (file-name-nondirectory (buffer-file-name)))))
    (with-temp-buffer
      (insert-buffer this-buffer)
      (cscsd-make-KeyPhrase keyphrase midifile durfile)
      (cscsd-set-Options "\n" cscsd-midi-options midifile "\n")
      ;; f0 & co..
      (unless (cscsd-set-macro-def "MidiPhraseDuration" 
				   (with-temp-buffer (insert-file-contents durfile) (buffer-string))
				   'sco) 
	(cscsd-goto-end-of-sco)
	(re-search-backward "^e" nil t) 
	(insert "\n\n")
	(forward-line -1)
	(insert "f0 ")
	(insert-file-contents durfile)) 
      (write-file csdfile)
      (set-cscsd-post-csound-log-action `(lambda () ;; nettoyage:
                                           (delete-file ,midifile)
                                           (delete-file ,durfile)
                                           (delete-file ,csdfile)
                                           (delete-file ,rootname)))
      (cscsd-process nil audioname)
      (setq audiofile cscsd-latest-audio-file))
    ;; informer le buffer source du nouveau fichier audio:
    (setq cscsd-latest-audio-file audiofile)))

(defun cscsd-process-midi-file (filename &optional audioname) ;; pas de contrle de la dure ???
  ""
  (let* ((rootname (make-temp-file "cscsd-tmp"))
	 (csdfile (concat rootname ".csd"))
	 (this-buffer (current-buffer)))
    (with-temp-buffer
      (insert-buffer this-buffer)
      (cscsd-set-Options "\n" cscsd-midi-options "\"" filename "\"\n")
      (write-file csdfile)
      (cscsd-process nil audioname)
      (delete-file csdfile)
      cscsd-latest-audio-file)))


(defun cscsd-make-KeyPhrase (keyphrase &optional midi-file txt-file k-file)
  "Invoke lowkey to process the KEYPHRASE area
if MIDI-FILE, save the resulting KeyPhrase in that file
if TXT-FILE, write KeyPhrase duration there
if K-FILE, save KeyPhrase there in KeyKit format"
  (save-excursion
    (goto-char (point-min))
    (copy-region-as-kill 
     (or (re-search-forward (concat "<KeyPhrase>[ \t]*#+" keyphrase "[ \t#]*$") nil t) ; backward compatibility
	 (re-search-forward (concat "<KeyPhrase label=\"" keyphrase "\">")))
     (progn
       (search-forward "</KeyPhrase>")
       (search-backward "</KeyPhrase>")))
    (with-temp-buffer
      (yank)
      (when midi-file
	(insert "\n\n"
		"writemf(KeyPhrase+'p0v0d" (number-to-string (* 192 cscsd-key-decay)) "',\"" midi-file "\")\n"))
      (when txt-file
	(insert "f=open(\"" txt-file "\",\"w\")\n"
		"put(f,string(" (number-to-string cscsd-key-decay) "+latest(KeyPhrase)/float(seconds(1))))\n"
		"close(f)\n"
		"\n"))
      (when k-file
	(insert "f=open(\"" k-file "\",\"w\")\n"
		"put(f,string(KeyPhrase))\n"
		"close(f)\n"
		"\n"))
      (kk-run-lowkey "" (point-min) (point-max)))))


(defun cscsd-KeyPhrase-to-CsMidifileB (keyphrase filename) ;;; pas besoin de filename !! voir avec temp
  (cscsd-make-KeyPhrase keyphrase filename)
  (cscsd-insert-midifile filename))

(defun cscsd-display-KeyPhrase (&optional keyphrase)
  (interactive "sKeyPhrase name> ")
  (kk-tcpip-display-phrase (get-KeyPhrase keyphrase)))

(defun get-KeyPhrase (keyphrase)
  "Invoke lowkey and return the KEYPHRASE defined in the corresponding area,
as a string containing its representation in KeyKit format"
  (let ((k-file (concat (make-temp-file "cscsd-KeyPhrase-") ".k")))
    (cscsd-make-KeyPhrase keyphrase nil nil k-file)
    (with-temp-buffer
      (insert-file-contents k-file)
      (delete-file k-file)
      (buffer-string))))

(defun get-MIDIfile (&optional file-name)
  (delete ?\n (lowkey-eval "print(onlynotes(readmf(\"" (or file-name (read-file-name "MIDI file: ")) "\")))")))
		    
(defun csound-csd-keyphrases-menu ()       
  "Dynamic submenu items for \"Csd\""
  (when (and csound-csd-mode
	   (featurep 'keykit-mode))
    (let (area (items '("KeyPhrases")))
      (save-excursion
	(goto-char (point-min))
	(while (re-search-forward "^[ \t]*<KeyPhrase label=\"\\(.*\\)\">" nil t)
	  (setq area (match-string 1))
	  (add-to-list 'items 		       
		       `(,area
			 "Process as MIDI input:"
			 ["with output to DAC"  (let ((cscsd-midi-options "-dodac -TF"))
                                                  (cscsd-process-KeyPhrase ,area)) t]
			 ["with default cscsd-midi-options"  (cscsd-process-KeyPhrase ,area) t]
			 "--"
			 "Keykit actions:"
			 ["display phrase in Group tool"
			  (keykit-do (format "DisplayInWgroup(%s)" (get-KeyPhrase ,area))) t]
			 ["snarf it" 
			  (keykit-do (format "Snarf=%s" (get-KeyPhrase ,area))) t]
			 ) t)))
      items)))
	

;; === this is it.
(provide 'csound-key)

;; csound-key.el ends here




